/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcxvcddisc.h"

#include "kdcfilefactory.h"
#include "lib/kdclib.h"

#include <QTreeWidgetItem>
#include <QString>
#include <QVector>
#include <QDrag>
#include <QUrl>
#include <QMimeData>

#include <KLocale>

KDCXVCDDisc::KDCXVCDDisc( KDC::DiscType type, int id, QWidget *parent )
	: KDCDisc( type, id, parent )
{

}

void KDCXVCDDisc::insertMovieGroup( int index, KDCMovieGroup *group )
{
	qDebug( "KDCXVCDDisc::insertMovieGroup( %d )::begin", KDCLib::isNull( group ) );
	insertTopLevelItem( index, group );
	++p_movieID;
	group->p_id = p_movieID;
	group->p_disc = this;
	qDebug( "KDCXVCDDisc::insertMovieGroup()::end" );
}

void KDCXVCDDisc::deleteMovie( KDCMovie *movie )
{
	qDebug( "KDCXVCDDisc::deleteMovie( %d )::begin", KDCLib::isNull( movie ) );
	if ( movie->movieGroup() )
		removeMovieFromGroup( movie );
	else
		invisibleRootItem()->removeChild( movie );
	delete movie;
	qDebug( "KDCXVCDDisc::deleteMovie()::end" );
}

void KDCXVCDDisc::destroyMovieGroup( KDCMovieGroup *group )
{
	qDebug( "KDCXVCDDisc::removeMovieGroup( %d )::begin", KDCLib::isNull( group ) );
	QList<QTreeWidgetItem*> movies = group->takeMovies();
	int index = indexOfTopLevelItem( group );
	deleteMovieGroup( group );
	invisibleRootItem()->insertChildren( index, movies );
	qDebug( "KDCXVCDDisc::removeMovieGroup()::end" );
}

void KDCXVCDDisc::removeMovieFromGroup( KDCMovie *movie )
{
	qDebug( "KDCXVCDDisc::removeMovieFromGroup( %d )::begin", KDCLib::isNull( movie ) );
	KDCMovieGroup *group = movie->movieGroup();
	group->removeMovie( movie );
	if ( group->count() == 1 )
		destroyMovieGroup( group );
	qDebug( "KDCXVCDDisc::removeMovieFromGroup()::end" );
}

void KDCXVCDDisc::deleteMovieGroup( KDCMovieGroup *group )
{
	qDebug( "KDCXVCDDisc::deleteMovieGroup( %d )::begin", KDCLib::isNull( group ) );
	invisibleRootItem()->removeChild( group );
	delete group;
	qDebug( "KDCXVCDDisc::deleteMovieGroup()::end" );
}

QStringList KDCXVCDDisc::checkErrors()
{
	KDCMovieGroup *group;
	QList<KDCMovie*> movies;
	QStringList errors;

	if ( topLevelItemCount() == 0 )
		errors << i18n("Disc is empty.");
	else
	{
		QTreeWidgetItemIterator it( this );
		while( *it )
		{
			if ( ( *it )->type() == KDC::Movie )
				movies.append( static_cast<KDCMovie*>( *it ) );
			else
				if ( ( *it )->type() == KDC::MovieGroup )
				{
					group = static_cast<KDCMovieGroup*>( *it );
					movies.append( group->movie( 0 ) );
					if ( group->hasErrors() )
						errors << i18n("Movies in clip ") + group->outputName() + i18n(" have different parameters.");
				}
			++it;
		}
	}
	for ( int i=0; i<movies.size(); ++i )
		for ( int j=i+1; j<movies.size(); ++j )
			if ( *( movies.at( i ) ) != *( movies.at( j ) ) )
			{
				QString str;
				str.append( i18n("Movies: ") );
				str.append( movies.at( i )->fileName() + i18n(" and ") + movies.at( j )->fileName() + " " );
				str.append( "have different parameters." );
				errors << str;
				break;
			}

	if ( !errors.isEmpty() )
		errors.prepend( i18n("Disc: ") + name() );
	return errors;
}

void KDCXVCDDisc::moveUp()
{
	qDebug( "KDCXVCDDisc::moveUp()::begin" );
	blockSignals( TRUE );
	QTreeWidgetItem *item = currentItem();
	bool expand = item->isExpanded();
	int index = indexOfTopLevelItem( item );
	takeTopLevelItem( index );
	insertTopLevelItem( index - 1, item );
	item->setExpanded( expand );
	blockSignals( FALSE );
	setCurrentItem( item );
	qDebug( "KDCXVCDDisc::moveUp()::end" );
}

void KDCXVCDDisc::moveDown()
{
	qDebug( "KDCXVCDDisc::moveDown()::begin" );
	blockSignals( TRUE );
	QTreeWidgetItem *item = currentItem();
	bool expand = item->isExpanded();
	int index = indexOfTopLevelItem( item );
	takeTopLevelItem( index );
	insertTopLevelItem( index + 1, item );
	item->setExpanded( expand );
	blockSignals( FALSE );
	setCurrentItem( item );
	qDebug( "KDCXVCDDisc::moveDown()::end" );
}

void KDCXVCDDisc::moveMovieToMovieGroup( KDCMovie *movie, KDCMovieGroup *group )
{
	qDebug( "KDCXVCDDisc::moveMovieToMovieGroup( %d, %d )::begin", KDCLib::isNull( movie ), KDCLib::isNull( group ) );
	if ( movie->movieGroup() )
		removeMovieFromGroup( movie );
	else
		invisibleRootItem()->removeChild( movie );
	group->addMovie( movie );
	qDebug( "KDCXVCDDisc::moveMovieToMovieGroup()::end" );
}

QList<QUrl> KDCXVCDDisc::codeItem( QTreeWidgetItem *item )
{
	QList<QUrl> list;
	if ( item->type() == KDC::MovieInfo )
		item = item->parent();
	list.append( QUrl( QString::number( indexOfTopLevelItem( item ) ) ) );
	list.append( QUrl( QString::number( item->isExpanded() ) ) );
	return list;
}

void KDCXVCDDisc::mouseMoveEvent( QMouseEvent *event )
{
	KDCDisc::mouseMoveEvent( event );
	// if not left button - return
	if ( !( event->buttons() & Qt::LeftButton ) )
		return;

	// if no item selected, return (else it would crash)
	if ( currentItem() == NULL )
		return;

	QDrag *drag = new QDrag( this );
	QMimeData *mimeData = new QMimeData;

	// construct list of QUrls
	mimeData->setUrls( codeItem( currentItem() ) );
	drag->setMimeData( mimeData );
	// start drag
	p_drag = TRUE;
	qDebug( "KDCXVCDDisc::mouseMoveEvent()::start dragging" );
	drag->start( Qt::CopyAction | Qt::MoveAction );
}

bool KDCXVCDDisc::dropMimeData( QTreeWidgetItem *parent, int index, const QMimeData *data, Qt::DropAction action )
{
	QList<QUrl> urlList;
	urlList = data->urls();
	bool res = TRUE;

	//if ( parent )
		//setCurrentItem( parent );
	qDebug( "KDCXVCDDisc::dropMimeData()::begin::parent = %d", KDCLib::isNull( parent ) );
	if ( !p_drag )
	{
		QStringList files;
		foreach( QUrl url, urlList )
			files.append( url.toLocalFile() );

		files.removeLast();
		qDebug( "KDCXVCDDisc::dropMimeData()::emit dropedSomeFiles()" );
		emit dropedSomeFiles( files );
	}
	else
		if ( !parent )
			res = FALSE;
	else
	{
		p_drag = FALSE;
		QList<QUrl> parentList = codeItem( parent );
		if ( urlList != parentList )
		{
			blockSignals( TRUE );
			qDebug( "KDCXVCDDisc::dropMimeData():: %d", urlList.count() );
			int index1 = urlList.first().toString().toInt();
			QTreeWidgetItem *item = topLevelItem( index1 );
			int index2 = parentList.first().toString().toInt();
			takeTopLevelItem( index1 );
			insertTopLevelItem( index2, item );
			blockSignals( FALSE );
			setCurrentItem( item );
			item->setExpanded( urlList.last().toString().toInt() );
		}
	}
	qDebug( "KDCXVCDDisc::dropMimeData()::end" );
	return res;
}

void KDCXVCDDisc::addMoviePrivate( KDCMovie *movie )
{
	++p_movieID;
	movie->p_id = p_movieID;
	movie->p_disc = this;
	QTreeWidgetItem *item = currentItem();
	if ( item )
	{
		int index = indexOfTopLevelItem( item );
		insertTopLevelItem( index, movie );
	}
	else
		addTopLevelItem( movie );
}
